
require "ptable"

local packing = {}
packing[3] = {[0]=
   72, 75, 68, 45, 66, 71, 77, 80,  5,
   73, 54, 50, 48, 47, 19, 53, 31,  8,
   76, 58, 42,  0, 25, 52,  3,  2, 14,
   63, 64, 36, 74, 57, 38, 59, 33, 62,
   46, 65, 17, 55, 56, 24, 26, 27, 21,
   78, 60, 40, 32, 28, 30,  1, 79, 35,
   67, 39, 44, 49, 22, 16, 43, 41, 10,
   69, 37, 23, 51, 18, 70, 15, 29, 12,
    6, 34, 11, 20,  9, 13,  4, 61,  7,
}


local function all_combinations (num_colors)
   return coroutine.wrap(function ()
      local i = 0
      local n = num_colors-1
      for ne = 0, n do
         for se = 0, n do
            for sw = 0, n do
               for nw = 0, n do
                  coroutine.yield(ne, se, sw, nw, i)
                  i = i + 1
               end
            end
         end
      end
   end)
end

local tileset = {}
tileset.__index = tileset

local floor = math.floor
local function new (cols, rows, num_colors, tilesize, tilesheet, tilesheet_cols)
   tilesheet_cols = tilesheet_cols or num_colors^2
   local t = setmetatable({
      x = 0,
      y = 0,
      cols=cols,
      rows=rows,
      num_colors=num_colors,
      tilesize=tilesize,
      tilesheet=tilesheet,
      toggled={},
      quads={}
   }, tileset)
   for ne, se, sw, nw, i in all_combinations(num_colors) do
      if packing[num_colors] then
         i = packing[num_colors][i]
      end
      local tilename = ne..se..sw..nw
      local x = (i % tilesheet_cols) * tilesize
      local y = floor(i / tilesheet_cols) * tilesize
      t.quads[tilename] = love.graphics.newQuad(x, y, tilesize, tilesize, tilesheet:getWidth(), tilesheet:getHeight())
   end
   return t
end

function tileset:set_tilesheet (tilesheet)
   self.tilesheet = tilesheet
   self.sb:setImage(tilesheet)
end

function tileset:move (x, y)
   self.x = self.x + x
   self.y = self.y + y
   self.sb = nil
end

function tileset:tile_at (x, y)
   local ts = self.tilesize
   x = floor((x+ts/2)/ts) + self.x
   y = floor((y+ts/2)/ts) + self.y
   return x, y
end

local function key (x, y)
   return x..":"..y
end

function tileset:toggle (x, y)
   local k = key(x, y)
   if self.toggled[k] then
      self.toggled[k] = nil
   else
      self.toggled[k] = true
   end
   self.sb = nil
end

function tileset:is_toggled (x, y)
   return self.toggled[key(x, y)]
end

function tileset:get_color (x, y)
   local h = hash(x, y)
   local show = h > 70
   if self:is_toggled(x, y) then
      show = not show
   end
   local color = h%2 + 1
   return show and color or 0
end

function tileset:generate_sprite_batch ()
   local c = {}
   for y = 0, self.rows do
      c[y] = {}
      for x = 0, self.cols do
         c[y][x] = self:get_color(x+self.x, y+self.y)
      end
   end
   local sb = love.graphics.newSpriteBatch(self.tilesheet, self.cols*self.rows)
   sb:bind()
   for y = 0, self.rows-1 do
      for x = 0, self.cols-1 do
         local tilename = c[y][x+1]..c[y+1][x+1]..c[y+1][x]..c[y][x]
         sb:addq(self.quads[tilename], x*self.tilesize, y*self.tilesize)
      end
   end
   sb:unbind()
   self.sb = sb
end

function tileset:draw (...)
   if not self.sb then
      self:generate_sprite_batch()
   end
   love.graphics.draw(self.sb, ...)
end

return setmetatable({new = new},
   {__call = function(_, ...) return new(...) end})
