
local G
local scale = 3
local ts = 16

function love.load ()
   love.keyboard.setKeyRepeat(0.1, 0.1)
   G = love.graphics
   G.setDefaultImageFilter("nearest", "nearest")
   DisplayTiles = require "display_corner_tile"
   local cols = G.getWidth()/ts/scale + 1
   local rows = G.getHeight()/ts/scale + 1
   cobbles = G.newImage "cobblestone.png"
   guide = G.newImage "guide.png"
   cobblestones = DisplayTiles(cols, rows, 3, ts, cobbles)
end

function love.draw ()
   cobblestones:draw(0, 0, 0, scale, scale)
end

function love.mousepressed (x, y)
   local x, y = cobblestones:tile_at(x/scale, y/scale)
   cobblestones:toggle(x, y)
   goal = cobblestones:get_color(x, y) ~= 0
   dragging = true
end

function love.update ()
   if dragging then
      local x, y = love.mouse.getPosition()
      x, y = cobblestones:tile_at(x/scale, y/scale)
      local color = cobblestones:get_color(x, y)
      local on = color ~= 0
      if on ~= goal then
         cobblestones:toggle(x, y)
      end
   end
end

function love.mousereleased ()
   dragging = false
end

local movement_keys = {
   up    = { 0, -1},
   down  = { 0,  1},
   left  = {-1,  0},
   right = { 1,  0},
   kp7   = {-1, -1},
   kp9   = { 1, -1},
   kp1   = {-1,  1},
   kp3   = { 1,  1},
}
movement_keys.kp8 = movement_keys.up
movement_keys.kp2 = movement_keys.down
movement_keys.kp4 = movement_keys.left
movement_keys.kp6 = movement_keys.right

function love.keypressed (key)
   if key == "v" then
      guide_toggle = not guide_toggle
      cobblestones:set_tilesheet(guide_toggle and guide or cobbles)
      return
   end

   if movement_keys[key] then
      cobblestones:move(unpack(movement_keys[key]))
   end
end
